@@ -28,16 +28,16 @@ namespace Gdk {
 
 	public class Event : GLib.IWrapper {
 
-		[DllImport("gdksharpglue-2")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern EventType gtksharp_gdk_event_get_event_type (IntPtr evt);
 
-		[DllImport("gdksharpglue-2")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_gdk_event_get_window (IntPtr evt);
 
-		[DllImport("gdksharpglue-2")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern sbyte gtksharp_gdk_event_get_send_event (IntPtr evt);
 
-		[DllImport("libgdk-win32-2.0-0.dll")]
+		[DllImport("libgdk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gdk_event_get_type ();
 
 		IntPtr raw;
@@ -53,9 +53,10 @@ namespace Gdk {
 			}
 		}
 
+		static GLib.GType _gtype = new GLib.GType (gdk_event_get_type ());
 		public static GLib.GType GType {
 			get {
-				return new GLib.GType (gdk_event_get_type ());
+				return _gtype;
 			}
 		}
 
@@ -77,6 +78,11 @@ namespace Gdk {
 			}
 		}
 
+		public static Event New (IntPtr raw)
+		{
+			return GetEvent (raw);
+		}
+
 		public static Event GetEvent (IntPtr raw)
 		{
 			if (raw == IntPtr.Zero)
@@ -128,6 +134,14 @@ namespace Gdk {
 				return new EventWindowState (raw);
 			case EventType.Setting:
 				return new EventSetting (raw);
+#if GTK_SHARP_2_6
+			case EventType.OwnerChange:
+				return new EventOwnerChange (raw);
+#endif
+#if GTK_SHARP_2_8
+			case EventType.GrabBroken:
+				return new EventGrabBroken (raw);
+#endif
 			case EventType.Map:
 			case EventType.Unmap:
 			case EventType.NoExpose:
