@@ -26,10 +26,10 @@ namespace Gdk {
 
 	public class EventProximity : Event {
 
-		[DllImport("gdksharpglue")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern uint gtksharp_gdk_event_proximity_get_time (IntPtr evt);
 
-		[DllImport("gdksharpglue")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_gdk_event_proximity_get_device (IntPtr evt);
 
 		public EventProximity (IntPtr raw) : base (raw) {} 
