@@ -26,10 +26,10 @@ namespace Gdk {
 
 	public class EventSetting : Event {
 
-		[DllImport("gdksharpglue")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern SettingAction gtksharp_gdk_event_setting_get_action (IntPtr evt);
 
-		[DllImport("gdksharpglue")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_gdk_event_setting_get_name (IntPtr evt);
 
 		public EventSetting (IntPtr raw) : base (raw) {} 
@@ -42,7 +42,7 @@ namespace Gdk {
 
 		public string Name {
 			get {
-				return Marshal.PtrToStringAnsi (gtksharp_gdk_event_setting_get_name (Handle));
+				return GLib.Marshaller.Utf8PtrToString (gtksharp_gdk_event_setting_get_name (Handle));
 			}
 		}
 	}
