@@ -26,24 +26,30 @@ namespace GtkSharp.Generation {
 	using System.Xml;
 
 	public class BoxedGen : StructBase {
-		
+
 		public BoxedGen (XmlElement ns, XmlElement elem) : base (ns, elem) {}
-		
+
 		public override void Generate (GenerationInfo gen_info)
 		{
+			Method copy = methods["Copy"] as Method;
+			methods.Remove ("Copy");
+			methods.Remove ("Free");
+
+			gen_info.CurrentType = Name;
+
 			StreamWriter sw = gen_info.Writer = gen_info.OpenStream (Name);
 			base.Generate (gen_info);
-			sw.WriteLine ("\t\t[DllImport(\"glibsharpglue\")]");
+			sw.WriteLine ("\t\t[DllImport(\"glibsharpglue-2\", CallingConvention = CallingConvention.Cdecl)]");
 			sw.WriteLine ("\t\tstatic extern IntPtr glibsharp_value_get_boxed (ref GLib.Value val);");
 			sw.WriteLine ();
-			sw.WriteLine ("\t\t[DllImport(\"glibsharpglue\")]");
+			sw.WriteLine ("\t\t[DllImport(\"glibsharpglue-2\", CallingConvention = CallingConvention.Cdecl)]");
 			sw.WriteLine ("\t\tstatic extern void glibsharp_value_set_boxed (ref GLib.Value val, ref " + QualifiedName + " boxed);");
 			sw.WriteLine ();
 			sw.WriteLine ("\t\tpublic static explicit operator GLib.Value (" + QualifiedName + " boxed)");
 			sw.WriteLine ("\t\t{");
 
 			sw.WriteLine ("\t\t\tGLib.Value val = GLib.Value.Empty;");
-			sw.WriteLine ("\t\t\tval.Init (" + QualifiedName + ".GType);");
+			sw.WriteLine ("\t\t\tval.Init (" + Name + "Attribute.GType);");
 			sw.WriteLine ("\t\t\tglibsharp_value_set_boxed (ref val, ref boxed);");
 			sw.WriteLine ("\t\t\treturn val;");
 			sw.WriteLine ("\t\t}");
@@ -55,14 +61,24 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t\treturn New (boxed_ptr);");
 			sw.WriteLine ("\t\t}");
 
+			if (copy != null && copy.IsDeprecated) {
+				sw.WriteLine ();
+				sw.WriteLine ("\t\t[Obsolete(\"This is a no-op\")]");
+				sw.WriteLine ("\t\tpublic " + QualifiedName + " Copy() {");
+				sw.WriteLine ("\t\t\treturn this;");
+				sw.WriteLine ("\t\t}");
+			}
+
 			sw.WriteLine ("#endregion");
                         AppendCustom(sw, gen_info.CustomDir);
                         sw.WriteLine ("\t}");
-                        sw.WriteLine ("}");
+			var method = GetMethod ("GetType") ?? GetMethod ("GetGType");
+			if (method != null)
+				AttributeHelper.Gen (sw, Name, LibraryName, method.CName);
+			sw.WriteLine ("}");
 			sw.Close ();
 			gen_info.Writer = null;
 			Statistics.BoxedCount++;
-		}		
+		}
 	}
 }
-
