@@ -49,6 +49,16 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public bool IsInternal {
+			get {
+				if (elem.HasAttribute ("internal")) {
+					string attr = elem.GetAttribute ("internal");
+					return attr == "1" || attr == "true";
+				}
+				return false;
+			}
+		}
+
 		public string LibraryName {
 			get {
 				return ns.GetAttribute ("library");
@@ -75,6 +85,8 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public abstract string DefaultValue { get; }
+
 		public string QualifiedName {
 			get {
 				return NS + "." + Name;
@@ -88,12 +100,17 @@ namespace GtkSharp.Generation {
 		}
 
 		protected void AppendCustom (StreamWriter sw, string custom_dir)
+		{
+			AppendCustom (sw, custom_dir, Name);
+		}
+
+		protected void AppendCustom (StreamWriter sw, string custom_dir, string type_name)
 		{
 			char sep = Path.DirectorySeparatorChar;
-			string custom = custom_dir + sep + Name + ".custom";
+			string custom = custom_dir + sep + type_name + ".custom";
 			if (File.Exists(custom)) {
 				sw.WriteLine ("#region Customized extensions");
-				sw.WriteLine ("#line 1 \"" + Name + ".custom\"");
+				sw.WriteLine ("#line 1 \"" + type_name + ".custom\"");
 				FileStream custstream = new FileStream(custom, FileMode.Open, FileAccess.Read);
 				StreamReader sr = new StreamReader(custstream);
 				sw.WriteLine (sr.ReadToEnd ());
@@ -116,6 +133,8 @@ namespace GtkSharp.Generation {
 			return CallByName (var);
 		}
 
+		public abstract bool Validate ();
+
 		public void Generate ()
 		{
 			GenerationInfo geninfo = new GenerationInfo (ns);
