@@ -2,7 +2,7 @@
 //
 // Author: Mike Kestner <mkestner@speakeasy.net>
 //
-// Copyright (c) 2001-2003 Mike Kestner 
+// Copyright (c) 2001-2003 Mike Kestner
 // Copyright (c) 2003-2005 Novell, Inc.
 // Copyright (c) 2007 Novell, Inc.
 //
@@ -53,7 +53,7 @@ namespace GtkSharp.Generation {
 
 		public string Name {
 			get {
-				return name; 
+				return name;
 			}
 			set {
 				name = value;
@@ -67,7 +67,7 @@ namespace GtkSharp.Generation {
 				Statistics.ThrottledCount++;
 				return false;
 			}
-			
+
 			if (!parms.Validate () || !retval.Validate ()) {
 				Console.Write (" in signal " + Name + " ");
 				Statistics.ThrottledCount++;
@@ -124,7 +124,7 @@ namespace GtkSharp.Generation {
                                         return Name + "Args";
                         }
                 }
-                                                                                                                        
+
                 private string EventArgsQualifiedName {
                         get {
                                 if (IsEventHandler)
@@ -133,7 +133,7 @@ namespace GtkSharp.Generation {
                                         return container_type.NS + "." + Name + "Args";
                         }
                 }
-                                                                                                                        
+
                 private string EventHandlerName {
                         get {
                                 if (IsEventHandler)
@@ -144,7 +144,7 @@ namespace GtkSharp.Generation {
                                         return Name + "Handler";
                         }
                 }
-                                                                                                                        
+
                 private string EventHandlerQualifiedName {
                         get {
                                 if (IsEventHandler)
@@ -191,7 +191,7 @@ namespace GtkSharp.Generation {
 				if (igen is BoxedGen)
 					return retval.CSType + ".GType";
 				if (igen is EnumGen)
-					return retval.CSType + "GType.GType";
+					return retval.CSType + "Attribute.GType";
 
 				switch (retval.CSType) {
 				case "bool":
@@ -249,7 +249,7 @@ namespace GtkSharp.Generation {
 			sw.WriteLine("\t\t\t} catch (Exception) {");
 			sw.WriteLine ("\t\t\t\tException ex = new Exception (\"args.RetVal or 'out' property unset or set to incorrect type in " + EventHandlerQualifiedName + " callback\");");
 			sw.WriteLine("\t\t\t\tGLib.ExceptionManager.RaiseUnhandledException (ex, true);");
-			
+
 			sw.WriteLine ("\t\t\t\t// NOTREACHED: above call doesn't return.");
 			sw.WriteLine ("\t\t\t\tthrow ex;");
 			sw.WriteLine("\t\t\t}");
@@ -260,7 +260,7 @@ namespace GtkSharp.Generation {
 			if (IsEventHandler)
 				return;
 
-			sw.WriteLine ("\t\t[GLib.CDeclCallback]");
+			sw.WriteLine ("\t\t[UnmanagedFunctionPointer (CallingConvention.Cdecl)]");
 			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + DelegateName + " (" + CallbackSig + ", IntPtr gch);");
 			sw.WriteLine ();
 			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + CallbackName + " (" + CallbackSig + ", IntPtr gch)");
@@ -297,7 +297,7 @@ namespace GtkSharp.Generation {
 			string ns = container_type.NS;
 
 			StreamWriter sw = gen_info.OpenStream (EventHandlerName);
-			
+
 			sw.WriteLine ("namespace " + ns + " {");
 			sw.WriteLine ();
 			sw.WriteLine ("\tusing System;");
@@ -365,7 +365,7 @@ namespace GtkSharp.Generation {
 					string call_parm = p.CallString;
 
 					if (p.IsUserData && parms.IsHidden (p) && !parms.HideData && (i == 1 || parms [i - 1].Scope != "notified")) {
-						call_parm = "IntPtr.Zero"; 
+						call_parm = "IntPtr.Zero";
 					}
 
 					result += ", " + call_parm;
@@ -419,7 +419,7 @@ namespace GtkSharp.Generation {
 			glue.WriteLine ("}");
 
 			StreamWriter sw = gen_info.Writer;
-			sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+			sw.WriteLine ("\t\t[DllImport (\"{0}\", CallingConvention = CallingConvention.Cdecl)]", gen_info.GluelibName);
 			sw.WriteLine ("\t\tstatic extern {0} {1} ({2});\n", retval.MarshalType, glue_name, parms.ImportSignature);
 			GenVMDeclaration (sw, null);
 			sw.WriteLine ("\t\t{");
@@ -442,42 +442,46 @@ namespace GtkSharp.Generation {
 				sw.WriteLine ("\t\t\tGLib.Value ret = new GLib.Value (" + ReturnGType + ");");
 
 			sw.WriteLine ("\t\t\tGLib.ValueArray inst_and_params = new GLib.ValueArray (" + parms.Count + ");");
-			sw.WriteLine ("\t\t\tGLib.Value[] vals = new GLib.Value [" + parms.Count + "];");
-			sw.WriteLine ("\t\t\tvals [0] = new GLib.Value (this);");
-			sw.WriteLine ("\t\t\tinst_and_params.Append (vals [0]);");
+			sw.WriteLine ("\t\t\tusing (var val0 = new GLib.Value (this)) {");
+			sw.WriteLine ("\t\t\t\tinst_and_params.Append (val0);");
 			string cleanup = "";
+			string indent = new string ('\t', 4);
 			for (int i = 1; i < parms.Count; i++) {
 				Parameter p = parms [i];
+				indent = new string ('\t', 3 + i);
 				if (p.PassAs != "") {
 					if (SymbolTable.Table.IsBoxed (p.CType)) {
 						if (p.PassAs == "ref")
-							sw.WriteLine ("\t\t\tvals [" + i + "] = new GLib.Value (" + p.Name + ");");
+							sw.WriteLine (indent + "using (var val" + i + " = new GLib.Value (" + p.Name + ")) {");
 						else
-							sw.WriteLine ("\t\t\tvals [" + i + "] = new GLib.Value ((GLib.GType)typeof (" + p.CSType + "));");
-						cleanup += "\t\t\t" + p.Name + " = (" + p.CSType + ") vals [" + i + "];\n";
+							sw.WriteLine (indent + "using (var val" + i + " = new GLib.Value ((GLib.GType)typeof (" + p.CSType + "))) {");
+						cleanup += indent + p.Name + " = (" + p.CSType + ") vals [" + i + "];\n";
 					} else {
 						if (p.PassAs == "ref")
-							sw.WriteLine ("\t\t\tIntPtr " + p.Name + "_ptr = GLib.Marshaller.StructureToPtrAlloc (" + p.Generatable.CallByName (p.Name) + ");");
+							sw.WriteLine (indent + "IntPtr " + p.Name + "_ptr = GLib.Marshaller.StructureToPtrAlloc (" + p.Generatable.CallByName (p.Name) + ");");
 						else
-							sw.WriteLine ("\t\t\tIntPtr " + p.Name + "_ptr = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (" + p.MarshalType + ")));");
+							sw.WriteLine (indent + "IntPtr " + p.Name + "_ptr = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (" + p.MarshalType + ")));");
 
-						sw.WriteLine ("\t\t\tvals [" + i + "] = new GLib.Value (" + p.Name + "_ptr);");
-						cleanup += "\t\t\t" + p.Name + " = " + p.FromNative ("(" + p.MarshalType + ") Marshal.PtrToStructure (" + p.Name + "_ptr, typeof (" + p.MarshalType + "))") + ";\n";
-						cleanup += "\t\t\tMarshal.FreeHGlobal (" + p.Name + "_ptr);\n";
+						sw.WriteLine (indent + "using (var val" + i + " = new GLib.Value (" + p.Name + "_ptr)) {");
+						cleanup += indent + p.Name + " = " + p.FromNative ("(" + p.MarshalType + ") Marshal.PtrToStructure (" + p.Name + "_ptr, typeof (" + p.MarshalType + "))") + ";\n";
+						cleanup += indent + "Marshal.FreeHGlobal (" + p.Name + "_ptr);\n";
 					}
 				} else if (p.IsLength && parms [i - 1].IsString)
-					sw.WriteLine ("\t\t\tvals [" + i + "] = new GLib.Value (System.Text.Encoding.UTF8.GetByteCount (" + parms [i-1].Name + "));");
+					sw.WriteLine (indent + "using (var val" + i + " = new GLib.Value (System.Text.Encoding.UTF8.GetByteCount (" + parms [i-1].Name + "))) {");
 				else
-					sw.WriteLine ("\t\t\tvals [" + i + "] = new GLib.Value (" + p.Name + ");");
+					sw.WriteLine (indent + "using (var val" + i + " = new GLib.Value (" + p.Name + ")) {");
 
-				sw.WriteLine ("\t\t\tinst_and_params.Append (vals [" + i + "]);");
+				sw.WriteLine (indent + "inst_and_params.Append (val" + i + ");");
 			}
 
-			sw.WriteLine ("\t\t\tg_signal_chain_from_overridden (inst_and_params.ArrayPtr, ref ret);");
+			sw.WriteLine (indent + "g_signal_chain_from_overridden (inst_and_params.ArrayPtr, ref ret);");
 			if (cleanup != "")
 				sw.WriteLine (cleanup);
-			sw.WriteLine ("\t\t\tforeach (GLib.Value v in vals)");
-			sw.WriteLine ("\t\t\t\tv.Dispose ();");
+			for (int i = 1; i < parms.Count; ++i) {
+				indent = new string ('\t', parms.Count - i + 3);
+				sw.WriteLine (indent + "}");
+			}
+			sw.WriteLine ("\t\t\t}");
 			if (!IsVoid) {
 				IGeneratable igen = SymbolTable.Table [retval.CType];
 				sw.WriteLine ("\t\t\t" + retval.CSType + " result = (" + (igen is EnumGen ? retval.CSType + ") (Enum" : retval.CSType) + ") ret;");
@@ -494,13 +498,13 @@ namespace GtkSharp.Generation {
 			bool use_glue = gen_info.GlueEnabled && implementor == null && ClassFieldName.Length > 0;
 			string glue_name = String.Empty;
 			ManagedCallString call = new ManagedCallString (parms, true);
-			sw.WriteLine ("\t\t[GLib.CDeclCallback]");
+			sw.WriteLine ("\t\t[UnmanagedFunctionPointer (CallingConvention.Cdecl)]");
 			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + Name + "VMDelegate (" + parms.ImportSignature + ");\n");
 
 			if (use_glue) {
 				glue = gen_info.GlueWriter;
 				glue_name = String.Format ("{0}sharp_{1}_override_{2}", container_type.NS.ToLower ().Replace (".", "_"), container_type.Name.ToLower (), ClassFieldName);
-				sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+				sw.WriteLine ("\t\t[DllImport (\"{0}\", CallingConvention = CallingConvention.Cdecl)]", gen_info.GluelibName);
 				sw.WriteLine ("\t\tstatic extern void {0} (IntPtr gtype, {1}VMDelegate cb);\n", glue_name, Name);
 				glue.WriteLine ("void {0} (GType gtype, gpointer cb);\n", glue_name);
 				glue.WriteLine ("void\n{0} (GType gtype, gpointer cb)", glue_name);
@@ -549,7 +553,7 @@ namespace GtkSharp.Generation {
 		public void GenEvent (StreamWriter sw, ClassBase implementor, string target)
 		{
 			string args_type = IsEventHandler ? "" : ", typeof (" + EventArgsQualifiedName + ")";
-			
+
 			if (Marshaled) {
 				GenCallback (sw);
 				args_type = ", new " + DelegateName + "(" + CallbackName + ")";
@@ -585,9 +589,8 @@ namespace GtkSharp.Generation {
 			else
 				GenChainVirtualMethod (sw, implementor);
 			GenEvent (sw, implementor, "this");
-			
+
 			Statistics.SignalCount++;
 		}
 	}
 }
-
