@@ -1,4 +1,4 @@
-// GtkSharp.Generation.StructBase.cs - The Structure/Object Base Class.
+// GtkSharp.Generation.StructBase.cs - The Structure/Boxed Base Class.
 //
 // Author: Mike Kestner <mkestner@speakeasy.net>
 //
@@ -27,9 +27,10 @@ namespace GtkSharp.Generation {
 	using System.Text.RegularExpressions;
 	using System.Xml;
 
-	public abstract class StructBase : ClassBase {
+	public abstract class StructBase : ClassBase, IManualMarshaler {
 	
-		ArrayList fields = new ArrayList ();
+		new ArrayList fields = new ArrayList ();
+		bool need_read_native = false;
 
 		protected StructBase (XmlElement ns, XmlElement elem) : base (ns, elem)
 		{
@@ -40,7 +41,7 @@ namespace GtkSharp.Generation {
 
 				switch (node.Name) {
 				case "field":
-					fields.Add (member);
+					fields.Add (new StructField (member, this));
 					break;
 
 				case "callback":
@@ -55,55 +56,48 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public override string MarshalType {
-			get
-			{
-				return "ref " + QualifiedName;
+		public override string DefaultValue {
+			get {
+				return QualifiedName + ".Zero";
 			}
 		}
 
-		public override string MarshalReturnType {
-			get
-			{
+		public override string MarshalType {
+			get {
 				return "IntPtr";
 			}
 		}
 
-		public override string ToNativeReturnType {
-			get
-			{
-				return QualifiedName;
-			}
-		}
-
-		public override string CallByName (string var_name)
-		{
-			return "ref " + var_name;
+		public override string AssignToName {
+			get { throw new NotImplementedException (); }
 		}
 
 		public override string CallByName ()
 		{
-			return "ref this";
+			return "this_as_native";
 		}
 
-		public override string AssignToName {
-			get { return "raw"; }
-		}
-
-		public override string FromNative(string var)
+		public override string CallByName (string var)
 		{
-			return var;
+			return var + "_as_native";
+		}
+
+		public override string FromNative (string var)
+		{
+			if (DisableNew)
+				return var + " == IntPtr.Zero ? " + QualifiedName + ".Zero : (" + QualifiedName + ") System.Runtime.InteropServices.Marshal.PtrToStructure (" + var + ", typeof (" + QualifiedName + "))";
+			else
+				return QualifiedName + ".New (" + var + ")";
 		}
 		
-		public override string FromNativeReturn(string var)
+		public string AllocNative (string var)
 		{
-			return QualifiedName + ".New (" + var + ")";
+			return "GLib.Marshaller.StructureToPtrAlloc (" + var + ")";
 		}
 
-		public override string ToNativeReturn(string var)
+		public string ReleaseNative (string var)
 		{
-			// FIXME
-			return var;
+			return "Marshal.FreeHGlobal (" +var + ")";
 		}
 
 		private bool DisableNew {
@@ -112,23 +106,38 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		protected void GenFields (StreamWriter sw)
+		protected new void GenFields (GenerationInfo gen_info)
 		{
-			Field.bitfields = 0;
+			int bitfields = 0;
 			bool need_field = true;
-			foreach (XmlElement field_elem in fields) {
-				Field field = new Field (field_elem);
-				if (field.IsBit) {
-					if (need_field)
+
+			foreach (StructField field in fields) {
+				if (field.IsBitfield) {
+					if (need_field) {
+						StreamWriter sw = gen_info.Writer;
+
+						sw.WriteLine ("\t\tprivate uint _bitfield{0};\n", bitfields++);
 						need_field = false;
-					else
-						continue;
+					}
 				} else
 					need_field = true;
-				field.Generate (sw);	
+				field.Generate (gen_info, "\t\t");
 			}
 		}
 
+		public override bool Validate ()
+		{
+			foreach (StructField field in fields) {
+				if (!field.Validate ()) {
+					Console.WriteLine ("in Struct " + QualifiedName);
+					if (!field.IsPointer)
+						return false;
+				}
+			}
+
+			return base.Validate ();
+		}
+
 		public override void Generate (GenerationInfo gen_info)
 		{
 			bool need_close = false;
@@ -147,14 +156,21 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ();
 			
 			sw.WriteLine ("#region Autogenerated code");
+			if (IsDeprecated)
+				sw.WriteLine ("\t[Obsolete]");
 			sw.WriteLine ("\t[StructLayout(LayoutKind.Sequential)]");
-			sw.WriteLine ("\tpublic struct " + Name + " {");
+			GenerateAttribute (sw);
+			string access = IsInternal ? "internal" : "public";
+			sw.WriteLine ("\t" + access + " struct " + Name + " {");
 			sw.WriteLine ();
 
-			GenFields (sw);
+			need_read_native = false;
+			GenFields (gen_info);
 			sw.WriteLine ();
 			GenCtors (gen_info);
-			GenMethods (gen_info, null, null);
+			GenMethods (gen_info, null, this);
+			if (need_read_native)
+				GenReadNative (sw);
 
 			if (!need_close)
 				return;
@@ -164,9 +180,16 @@ namespace GtkSharp.Generation {
 			
 			sw.WriteLine ("\t}");
 			sw.WriteLine ("}");
+
 			sw.Close ();
 			gen_info.Writer = null;
 		}
+
+		protected virtual void GenerateAttribute (StreamWriter writer)
+		{
+			if (GetMethod ("GetType") != null || GetMethod ("GetGType") != null)
+				writer.WriteLine ("\t[{0}]", Name);
+		}
 		
 		protected override void GenCtors (GenerationInfo gen_info)
 		{
@@ -176,12 +199,9 @@ namespace GtkSharp.Generation {
 			sw.WriteLine();
 			if (!DisableNew) {
 				sw.WriteLine ("\t\tpublic static " + QualifiedName + " New(IntPtr raw) {");
-				sw.WriteLine ("\t\t\tif (raw == IntPtr.Zero) {");
+				sw.WriteLine ("\t\t\tif (raw == IntPtr.Zero)");
 				sw.WriteLine ("\t\t\t\treturn {0}.Zero;", QualifiedName);
-				sw.WriteLine ("\t\t\t}");
-				sw.WriteLine ("\t\t\t{0} self = new {0}();", QualifiedName);
-				sw.WriteLine ("\t\t\tself = ({0}) Marshal.PtrToStructure (raw, self.GetType ());", QualifiedName);
-				sw.WriteLine ("\t\t\treturn self;");
+				sw.WriteLine ("\t\t\treturn ({0}) Marshal.PtrToStructure (raw, typeof ({0}));", QualifiedName);
 				sw.WriteLine ("\t\t}");
 				sw.WriteLine ();
 			}
@@ -192,6 +212,27 @@ namespace GtkSharp.Generation {
 			base.GenCtors (gen_info);
 		}
 
+		void GenReadNative (StreamWriter sw)
+		{
+			sw.WriteLine ("\t\tstatic void ReadNative (IntPtr native, ref {0} target)", QualifiedName);
+			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\ttarget = New (native);");
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ();
+		}
+
+		public override void Prepare (StreamWriter sw, string indent)
+		{
+			sw.WriteLine (indent + "IntPtr this_as_native = System.Runtime.InteropServices.Marshal.AllocHGlobal (System.Runtime.InteropServices.Marshal.SizeOf (this));");
+			sw.WriteLine (indent + "System.Runtime.InteropServices.Marshal.StructureToPtr (this, this_as_native, false);");
+		}
+
+		public override void Finish (StreamWriter sw, string indent)
+		{
+			need_read_native = true;
+			sw.WriteLine (indent + "ReadNative (this_as_native, ref this);");
+			sw.WriteLine (indent + "System.Runtime.InteropServices.Marshal.FreeHGlobal (this_as_native);");
+		}
 	}
 }
 
