@@ -27,7 +27,7 @@ namespace GLib {
 
 		IntPtr handle;
 
-		[DllImport("libglib-2.0-0.dll")]
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr g_string_free (IntPtr mem, bool free_segments);
 
 		~GString ()
@@ -35,12 +35,14 @@ namespace GLib {
 			g_string_free (handle, true);
 		}
 
-		[DllImport("libglib-2.0-0.dll")]
-		static extern IntPtr g_string_new (string text);
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		static extern IntPtr g_string_new (IntPtr text);
 
 		public GString (string text) 
 		{
-			handle = g_string_new (text);
+			IntPtr native_text = Marshaller.StringToPtrGStrdup (text);
+			handle = g_string_new (native_text);
+			Marshaller.Free (native_text);
 		}
 
 		public IntPtr Handle {
@@ -51,7 +53,7 @@ namespace GLib {
 		
 		public static string PtrToString (IntPtr ptr) 
 		{
-			return Marshal.PtrToStringAnsi (ptr);
+			return Marshaller.Utf8PtrToString (ptr);
 		}
 	}
 }
