@@ -6,7 +6,7 @@
 // Copyright (c) 2004-2005 Novell, Inc.
 //
 // This program is free software; you can redistribute it and/or
-// modify it under the terms of version 2 of the Lesser GNU General 
+// modify it under the terms of version 2 of the Lesser GNU General
 // Public License as published by the Free Software Foundation.
 //
 // This program is distributed in the hope that it will be useful,
@@ -24,6 +24,7 @@ namespace GLib {
 
 	using System;
 	using System.Collections;
+	using System.Collections.Generic;
 	using System.ComponentModel;
 	using System.Reflection;
 	using System.Runtime.InteropServices;
@@ -34,17 +35,20 @@ namespace GLib {
 		IntPtr handle;
 		ToggleRef tref;
 		bool disposed = false;
+		bool owned = true;
 		Hashtable data;
-		static Hashtable Objects = new Hashtable();
-		static ArrayList PendingDestroys = new ArrayList ();
+		static Dictionary<IntPtr, ToggleRef> Objects = new Dictionary<IntPtr, ToggleRef>();
+		static object lockObject = new object ();
+		static List<ToggleRef> PendingDestroys = new List<ToggleRef> ();
 		static bool idle_queued;
 
 		~Object ()
 		{
-			lock (PendingDestroys) {
+			lock (lockObject) {
 				lock (Objects) {
-					if (Objects[Handle] is ToggleRef)
-						PendingDestroys.Add (Objects [Handle]);
+					ToggleRef res;
+					if (Objects.TryGetValue (handle, out res))
+						PendingDestroys.Add (res);
 					Objects.Remove (Handle);
 				}
 				if (!idle_queued){
@@ -54,17 +58,16 @@ namespace GLib {
 			}
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_object_unref (IntPtr raw);
-		
+
 		static bool PerformQueuedUnrefs ()
 		{
-			object [] references;
+			List<ToggleRef> references;
 
-			lock (PendingDestroys){
-				references = new object [PendingDestroys.Count];
-				PendingDestroys.CopyTo (references, 0);
-				PendingDestroys.Clear ();
+			lock (lockObject) {
+				references = PendingDestroys;
+				PendingDestroys = new List<ToggleRef> ();
 				idle_queued = false;
 			}
 
@@ -80,8 +83,11 @@ namespace GLib {
 				return;
 
 			disposed = true;
-			ToggleRef toggle_ref = Objects [Handle] as ToggleRef;
-			Objects.Remove (Handle);
+			ToggleRef toggle_ref;
+			lock(Objects) {
+				Objects.TryGetValue (Handle, out toggle_ref);
+				Objects.Remove (Handle);
+			}
 			try {
 				if (toggle_ref != null)
 					toggle_ref.Free ();
@@ -89,12 +95,29 @@ namespace GLib {
 				Console.WriteLine ("Exception while disposing a " + this + " in Gtk#");
 				throw e;
 			}
+			if (owned)
+				g_object_unref (handle);
 			handle = IntPtr.Zero;
 			GC.SuppressFinalize (this);
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
-		static extern void g_object_ref (IntPtr raw);
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		static extern IntPtr g_object_ref (IntPtr raw);
+
+		public static Object TryGetObject (IntPtr o)
+		{
+			if (o == IntPtr.Zero)
+				return null;
+
+			ToggleRef tr;
+			lock(Objects)
+				Objects.TryGetValue (o, out tr);
+			if (tr != null && tr.IsAlive) {
+				return tr.Target;
+			}
+
+			return null;
+		}
 
 		public static Object GetObject(IntPtr o, bool owned_ref)
 		{
@@ -102,28 +125,23 @@ namespace GLib {
 				return null;
 
 			Object obj = null;
+			ToggleRef toggle_ref = null;
+			lock(Objects)
+				Objects.TryGetValue (o, out toggle_ref);
 
-			if (Objects.Contains (o)) {
-				ToggleRef toggle_ref = Objects [o] as ToggleRef;
-				if (toggle_ref != null && toggle_ref.IsAlive)
-					obj = toggle_ref.Target;
-			}
+			if (toggle_ref != null && toggle_ref.IsAlive)
+				obj = toggle_ref.Target;
 
-			if (obj != null && obj.Handle == o) {
-				if (owned_ref)
-					g_object_unref (obj.Handle);
+			if (obj != null && obj.Handle == o)
 				return obj;
-			}
 
-			if (!owned_ref)
-				g_object_ref (o);
-
-			obj = GLib.ObjectManager.CreateObject(o); 
+			obj = GLib.ObjectManager.CreateObject (o);
 			if (obj == null) {
 				g_object_unref (o);
 				return null;
 			}
 
+			obj.owned = owned_ref;
 			return obj;
 		}
 
@@ -148,7 +166,7 @@ namespace GLib {
 					break;
 				}
 			}
-					
+
 		}
 
 		private static void InvokeClassInitializers (GType gtype, System.Type t)
@@ -164,17 +182,106 @@ namespace GLib {
 			}
 
 			for (Type curr = t; curr != typeof(GLib.Object); curr = curr.BaseType) {
- 
+
 				if (curr.Assembly.IsDefined (typeof (IgnoreClassInitializersAttribute), false))
 					continue;
- 
+
 				foreach (MethodInfo minfo in curr.GetMethods(flags))
 					if (minfo.IsDefined (typeof (ClassInitializerAttribute), true))
 						minfo.Invoke (null, parms);
 			}
  		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		//  Key: The pointer to the ParamSpec of the property
+		//  Value: The corresponding PropertyInfo object
+		static Hashtable properties;
+		static Hashtable Properties {
+			get {
+				if (properties == null)
+					properties = new Hashtable ();
+				return properties;
+			}
+		}
+
+		[DllImport ("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
+		static extern void gtksharp_override_property_handlers (IntPtr type, GetPropertyDelegate get_cb, SetPropertyDelegate set_cb);
+
+		[DllImport ("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
+		static extern IntPtr gtksharp_register_property (IntPtr type, IntPtr name, IntPtr nick, IntPtr blurb, uint property_id, IntPtr property_type, bool can_read, bool can_write);
+
+		static void AddProperties (GType gtype, System.Type t)
+		{
+			uint idx = 1;
+
+			bool handlers_overridden = false;
+			foreach (PropertyInfo pinfo in t.GetProperties (BindingFlags.Instance | BindingFlags.Public | BindingFlags.DeclaredOnly)) {
+				foreach (object attr in pinfo.GetCustomAttributes (typeof (PropertyAttribute), false)) {
+					if(pinfo.GetIndexParameters().Length > 0)
+						throw(new InvalidOperationException(String.Format("GLib.RegisterPropertyAttribute cannot be applied to property {0} of type {1} because the property expects one or more indexed parameters", pinfo.Name, t.FullName)));
+
+					PropertyAttribute property_attr = attr as PropertyAttribute;
+					if (!handlers_overridden) {
+						gtksharp_override_property_handlers (gtype.Val, GetPropertyHandler, SetPropertyHandler);
+						handlers_overridden = true;
+					}
+
+					IntPtr native_name = GLib.Marshaller.StringToPtrGStrdup (property_attr.Name);
+					IntPtr native_nick = GLib.Marshaller.StringToPtrGStrdup (property_attr.Nickname);
+					IntPtr native_blurb = GLib.Marshaller.StringToPtrGStrdup (property_attr.Blurb);
+
+					IntPtr param_spec = gtksharp_register_property (gtype.Val, native_name, native_nick, native_blurb, idx, ((GType) pinfo.PropertyType).Val, pinfo.CanRead, pinfo.CanWrite);
+
+					GLib.Marshaller.Free (native_name);
+					GLib.Marshaller.Free (native_nick);
+					GLib.Marshaller.Free (native_blurb);
+
+					if (param_spec == IntPtr.Zero)
+						// The GType of the property is not supported
+						throw new InvalidOperationException (String.Format ("GLib.PropertyAttribute cannot be applied to property {0} of type {1} because the return type of the property is not supported", pinfo.Name, t.FullName));
+
+					Properties.Add (param_spec, pinfo);
+					idx++;
+				}
+			}
+		}
+
+		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
+		delegate void GetPropertyDelegate (IntPtr GObject, uint property_id, ref GLib.Value value, IntPtr pspec);
+
+		static void GetPropertyCallback (IntPtr handle, uint property_id, ref GLib.Value value, IntPtr param_spec)
+		{
+			GLib.Object obj = GLib.Object.GetObject (handle, false);
+			value.Val = (Properties [param_spec] as PropertyInfo).GetValue (obj, new object [0]);
+		}
+
+		static GetPropertyDelegate get_property_handler;
+		static GetPropertyDelegate GetPropertyHandler {
+			get {
+				if (get_property_handler == null)
+					get_property_handler = new GetPropertyDelegate (GetPropertyCallback);
+				return get_property_handler;
+			}
+		}
+
+		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
+		delegate void SetPropertyDelegate (IntPtr GObject, uint property_id, ref GLib.Value value, IntPtr pspec);
+
+		static void SetPropertyCallback(IntPtr handle, uint property_id, ref GLib.Value value, IntPtr param_spec)
+		{
+			GLib.Object obj = GLib.Object.GetObject (handle, false);
+			(Properties [param_spec] as PropertyInfo).SetValue (obj, value.Val, new object [0]);
+		}
+
+		static SetPropertyDelegate set_property_handler;
+		static SetPropertyDelegate SetPropertyHandler {
+			get {
+				if (set_property_handler == null)
+					set_property_handler = new SetPropertyDelegate (SetPropertyCallback);
+				return set_property_handler;
+			}
+		}
+
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_type_add_interface_static (IntPtr gtype, IntPtr iface_type, ref GInterfaceInfo info);
 
 		static void AddInterfaces (GType gtype, Type t)
@@ -185,14 +292,13 @@ namespace GLib {
 
 				GInterfaceAttribute attr = iface.GetCustomAttributes (typeof (GInterfaceAttribute), false) [0] as GInterfaceAttribute;
 				GInterfaceAdapter adapter = Activator.CreateInstance (attr.AdapterType, null) as GInterfaceAdapter;
-				
+
 				GInterfaceInfo info = adapter.Info;
 				g_type_add_interface_static (gtype.Val, adapter.GType.Val, ref info);
-				Console.WriteLine (adapter);
 			}
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_register_type (IntPtr name, IntPtr parent_type);
 
 		static int type_uid;
@@ -228,6 +334,7 @@ namespace GLib {
 			GType gtype = new GType (gtksharp_register_type (native_name, parent_gtype.Val));
 			GLib.Marshaller.Free (native_name);
 			GLib.GType.Register (gtype, t);
+			AddProperties (gtype, t);
 			ConnectDefaultHandlers (gtype, t);
 			InvokeClassInitializers (gtype, t);
 			AddInterfaces (gtype, t);
@@ -236,7 +343,7 @@ namespace GLib {
 		}
 
 
-		static Hashtable g_types = new Hashtable ();
+		static Dictionary<Type, GType> g_types = new Dictionary<Type, GType> ();
 
 		protected GType LookupGType ()
 		{
@@ -245,13 +352,24 @@ namespace GLib {
 
 		protected internal static GType LookupGType (System.Type t)
 		{
-			if (g_types.Contains (t))
-				return (GType) g_types [t];
-			
+			GType res;
+			if (g_types.TryGetValue (t, out res))
+				return res;
+
+			GTypeTypeAttribute geattr;
+			if ((geattr = (GTypeTypeAttribute)Attribute.GetCustomAttribute (t, typeof (GTypeTypeAttribute), false)) != null) {
+				var val = geattr.Type;
+				g_types [t] = val;
+				return val;
+			}
+
 			PropertyInfo pi = t.GetProperty ("GType", BindingFlags.DeclaredOnly | BindingFlags.Static | BindingFlags.Public);
-			if (pi != null)
-				return (GType) pi.GetValue (null, null);
-			
+			if (pi != null) {
+				var val = (GType)pi.GetValue (null, null);
+				g_types [t] = val;
+				return val;
+			}
+
 			return RegisterGType (t);
 		}
 
@@ -265,7 +383,7 @@ namespace GLib {
 			CreateNativeObject (new string [0], new GLib.Value [0]);
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr g_object_new (IntPtr gtype, IntPtr dummy);
 
 		[Obsolete]
@@ -274,17 +392,29 @@ namespace GLib {
 			Raw = g_object_new (gtype.Val, IntPtr.Zero);
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_object_newv (IntPtr gtype, int n_params, IntPtr[] names, GLib.Value[] vals);
 
 		protected virtual void CreateNativeObject (string[] names, GLib.Value[] vals)
 		{
-			IntPtr[] native_names = new IntPtr [names.Length];
-			for (int i = 0; i < names.Length; i++)
+			CreateNativeObject (names, vals, names.Length);
+		}
+
+		protected void CreateNativeObject (string [] names, GLib.Value [] vals, int count)
+		{
+			IntPtr[] native_names = new IntPtr [count];
+			for (int i = 0; i < count; i++)
 				native_names [i] = GLib.Marshaller.StringToPtrGStrdup (names [i]);
-			Raw = gtksharp_object_newv (LookupGType ().Val, names.Length, native_names, vals);
-			foreach (IntPtr p in native_names)
-				GLib.Marshaller.Free (p);
+			CreateNativeObject (native_names, vals, count);
+		}
+
+		protected void CreateNativeObject (IntPtr [] native_names, GLib.Value [] vals, int count)
+		{
+			Raw = gtksharp_object_newv (LookupGType ().Val, count, native_names, vals);
+			for (int i = 0; i < count; ++i) {
+				GLib.Marshaller.Free (native_names [i]);
+				vals [i].Dispose ();
+			}
 		}
 
 		protected virtual IntPtr Raw {
@@ -295,20 +425,22 @@ namespace GLib {
 				if (handle == value)
 					return;
 
-				if (handle != IntPtr.Zero) {
-					Objects.Remove (handle);
-					if (tref != null) {
-						tref.Free ();
-						tref = null;
+				lock(Objects) {
+					if (handle != IntPtr.Zero) {
+						Objects.Remove (handle);
+						if (tref != null) {
+							tref.Free ();
+							tref = null;
+						}
+					}
+					handle = value;
+					if (value != IntPtr.Zero) {
+						tref = new ToggleRef (this);
+						Objects [value] = tref;
 					}
 				}
-				handle = value;
-				if (value != IntPtr.Zero) {
-					tref = new ToggleRef (this);
-					Objects [value] = tref;
-				}
 			}
-		}	
+        	}
 
 		public static GLib.GType GType {
 			get {
@@ -316,7 +448,7 @@ namespace GLib {
 			}
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_get_type_name (IntPtr raw);
 
 		protected string TypeName {
@@ -343,9 +475,15 @@ namespace GLib {
 			}
 		}
 
+		public IntPtr OwnedHandle {
+			get {
+				return g_object_ref (handle);
+			}
+		}
+
 		Hashtable before_signals;
 		[Obsolete ("Replaced by GLib.Signal marshaling mechanism.")]
-		protected Hashtable BeforeSignals {
+		protected internal Hashtable BeforeSignals {
 			get {
 				if (before_signals == null)
 					before_signals = new Hashtable ();
@@ -355,7 +493,7 @@ namespace GLib {
 
 		Hashtable after_signals;
 		[Obsolete ("Replaced by GLib.Signal marshaling mechanism.")]
-		protected Hashtable AfterSignals {
+		protected internal Hashtable AfterSignals {
 			get {
 				if (after_signals == null)
 					after_signals = new Hashtable ();
@@ -383,7 +521,7 @@ namespace GLib {
 			}
 		}
 
-		[CDeclCallback]
+		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
 		delegate void NotifyDelegate (IntPtr handle, IntPtr pspec, IntPtr gch);
 
 		void NotifyCallback (IntPtr handle, IntPtr pspec, IntPtr gch)
@@ -441,10 +579,10 @@ namespace GLib {
 		}
 
 		public Hashtable Data {
-			get { 
+			get {
 				if (data == null)
 					data = new Hashtable ();
-				
+
 				return data;
 			}
 		}
@@ -454,24 +592,24 @@ namespace GLib {
 			get {
 				if (persistent_data == null)
 					persistent_data = new Hashtable ();
-				
+
 				return persistent_data;
 			}
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_object_get_property (IntPtr obj, IntPtr name, ref GLib.Value val);
 
 		protected GLib.Value GetProperty (string name)
 		{
-			Value val = new Value (this, name);
 			IntPtr native_name = GLib.Marshaller.StringToPtrGStrdup (name);
+			Value val = new Value (this, native_name);
 			g_object_get_property (Raw, native_name, ref val);
 			GLib.Marshaller.Free (native_name);
 			return val;
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_object_set_property (IntPtr obj, IntPtr name, ref GLib.Value val);
 
 		protected void SetProperty (string name, GLib.Value val)
@@ -481,7 +619,17 @@ namespace GLib {
 			GLib.Marshaller.Free (native_name);
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		static extern void g_object_notify (IntPtr obj, IntPtr property_name);
+
+		protected void Notify (string property_name)
+		{
+			IntPtr native_name = GLib.Marshaller.StringToPtrGStrdup (property_name);
+			g_object_notify (Handle, native_name);
+			GLib.Marshaller.Free (native_name);
+		}
+
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern void gtksharp_override_virtual_method (IntPtr gtype, IntPtr name, Delegate cb);
 
 		protected static void OverrideVirtualMethod (GType gtype, string name, Delegate cb)
@@ -491,10 +639,10 @@ namespace GLib {
 			GLib.Marshaller.Free (native_name);
 		}
 
-		[DllImport("libgobject-2.0-0.dll")]
+		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		protected static extern void g_signal_chain_from_overridden (IntPtr args, ref GLib.Value retval);
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern bool gtksharp_is_object (IntPtr obj);
 
 		internal static bool IsObject (IntPtr obj)
@@ -502,7 +650,7 @@ namespace GLib {
 			return gtksharp_is_object (obj);
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int gtksharp_object_get_ref_count (IntPtr obj);
 
 		protected int RefCount {
@@ -510,5 +658,16 @@ namespace GLib {
 				return gtksharp_object_get_ref_count (Handle);
 			}
 		}
+
+		internal void Harden ()
+		{
+			tref.Harden ();
+		}
+
+		static Object ()
+		{
+			if (Environment.GetEnvironmentVariable ("GTK_SHARP_DEBUG") != null)
+				GLib.Log.SetLogHandler ("GLib-GObject", GLib.LogLevelFlags.All, new GLib.LogFunc (GLib.Log.PrintTraceLogFunction));
+		}
 	}
 }
