@@ -31,24 +31,40 @@ namespace GLib {
 	using System.ComponentModel;
 	using System.Runtime.InteropServices;
 
-	public class Opaque : IWrapper {
+	public class Opaque : IWrapper, IDisposable {
 
 		IntPtr _obj;
-		static Hashtable Opaques = new Hashtable();
+		bool owned;
 
-		public static Opaque GetOpaque(IntPtr o)
+		[Obsolete ("Use more explicit overload.  This method always returns null")]
+		public static Opaque GetOpaque (IntPtr o)
 		{
-			WeakReference reference = (WeakReference) Opaques[(int)o];
-			if (reference == null || !reference.IsAlive)
-				return null;
+			return null;
+		}
 
-			return (Opaque) reference.Target;
+		public static Opaque GetOpaque (IntPtr o, Type type, bool owned)
+		{
+			Opaque opaque = (Opaque)Activator.CreateInstance (type, new object[] { o });
+			if (owned) {
+				if (opaque.owned) {
+					// The constructor took a Ref it shouldn't have, so undo it
+					opaque.Unref (o);
+				}
+				opaque.owned = true;
+			} else 
+				opaque = opaque.Copy (o);
+
+			return opaque;
   		}
   
-		public Opaque () {}
+		public Opaque ()
+		{
+			owned = true;
+		}
 
 		public Opaque (IntPtr raw)
 		{
+			owned = false;
 			Raw = raw;
 		}
 
@@ -57,17 +73,64 @@ namespace GLib {
 				return _obj;
 			}
 			set {
-				Opaques [value] = new WeakReference (this);
+				if (_obj != IntPtr.Zero) {
+					Unref (_obj);
+					if (owned)
+						Free (_obj);
+				}
 				_obj = value;
+				if (_obj != IntPtr.Zero) {
+					Ref (_obj);
+				}
 			}
 		}       
 
+		~Opaque ()
+		{
+			// for compat.  All subclasses should have
+			// generated finalizers if needed now.
+		}
+
+		public virtual void Dispose ()
+		{
+			Raw = IntPtr.Zero;
+			GC.SuppressFinalize (this);
+		}
+
+		// These take an IntPtr arg so we don't get conflicts if we need
+		// to have an "[Obsolete] public void Ref ()"
+
+		protected virtual void Ref (IntPtr raw) {}
+		protected virtual void Unref (IntPtr raw) {}
+		protected virtual void Free (IntPtr raw) {}
+		protected virtual Opaque Copy (IntPtr raw) 
+		{
+			return this;
+		}
+
 		public IntPtr Handle {
 			get {
 				return _obj;
 			}
 		}
 
+		public IntPtr OwnedCopy {
+			get {
+				Opaque result = Copy (Handle);
+				result.Owned = false;
+				return result.Handle;
+			}
+		}
+
+		public bool Owned {
+			get {
+				return owned;
+			}
+			set {
+				owned = value;
+			}
+		}
+
 		public override bool Equals (object o)
 		{
 			if (!(o is Opaque))
