@@ -27,13 +27,36 @@ namespace GLib
 	public class Thread
 	{
 		private Thread () {}
-		
-		[DllImport("libgthread-2.0-0.dll")]
+
+#if DISABLE_GTHREAD_CHECK
+		public static void Init ()
+		{
+			// GLib automatically inits threads in 2.31 and above
+			// http://developer.gnome.org/glib/unstable/glib-Deprecated-Thread-APIs.html#g-thread-init
+		}
+
+		public static bool Supported {
+			get { return true; }
+		}
+#else
+		[DllImport("libgthread-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_thread_init (IntPtr i);
 
 		public static void Init ()
 		{
 			g_thread_init (IntPtr.Zero);
 		}
+
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
+		static extern bool glibsharp_g_thread_supported ();
+
+		public static bool Supported
+		{
+			get {
+				return glibsharp_g_thread_supported ();
+			}
+		}
+#endif
+
 	}
 }
