@@ -5,7 +5,7 @@
 // Copyright (c) 2002 Mike Kestner
 //
 // This program is free software; you can redistribute it and/or
-// modify it under the terms of version 2 of the Lesser GNU General 
+// modify it under the terms of version 2 of the Lesser GNU General
 // Public License as published by the Free Software Foundation.
 //
 // This program is distributed in the hope that it will be useful,
@@ -28,8 +28,8 @@ namespace GLib {
 
 	public class Timeout {
 
-		[CDeclCallback]
-		delegate bool TimeoutHandlerInternal ();
+		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
+		delegate bool TimeoutHandlerInternal (IntPtr ptr);
 
 		internal class TimeoutProxy : SourceProxy {
 			public TimeoutProxy (TimeoutHandler real)
@@ -62,14 +62,17 @@ namespace GLib {
 					Source.Remove (ID);
 			}
 
-			public bool Handler ()
+			static bool Handler (IntPtr data)
 			{
 				try {
-					TimeoutHandler timeout_handler = (TimeoutHandler) real_handler;
+					SourceProxy obj;
+					lock (proxies)
+						obj = proxies [(int)data];
+					TimeoutHandler timeout_handler = (TimeoutHandler)obj.real_handler;
 
 					bool cont = timeout_handler ();
 					if (!cont)
-						Remove ();
+						obj.Remove ();
 					return cont;
 				} catch (Exception e) {
 					ExceptionManager.RaiseUnhandledException (e, false);
@@ -77,16 +80,17 @@ namespace GLib {
 				return false;
 			}
 		}
-		
-		private Timeout () {} 
-		[DllImport("libglib-2.0-0.dll")]
+
+		private Timeout () {}
+
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern uint g_timeout_add (uint interval, TimeoutHandlerInternal d, IntPtr data);
 
 		public static uint Add (uint interval, TimeoutHandler hndlr)
 		{
 			TimeoutProxy p = new TimeoutProxy (hndlr);
 
-			p.ID = g_timeout_add (interval, (TimeoutHandlerInternal) p.proxy_handler, IntPtr.Zero);
+			p.ID = g_timeout_add (interval, (TimeoutHandlerInternal) p.proxy_handler, (IntPtr)p.proxyId);
 			lock (Source.source_handlers)
 				Source.source_handlers [p.ID] = p;
 
@@ -94,4 +98,3 @@ namespace GLib {
 		}
 	}
 }
-
