@@ -1,8 +1,8 @@
 // TreeNodeAttribute.cs - Attribute to specify TreeNode information for a class
 //
-// Author: Mike Kestner  <mkestner@ximian.com>
+// Author: Mike Kestner  <mkestner@novell.com>
 //
-// Copyright (c) 2003 Novell, Inc.
+// Copyright (c) 2003-2005 Novell, Inc.
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of version 2 of the Lesser GNU General 
@@ -25,14 +25,20 @@ namespace Gtk {
 
 	[AttributeUsage(AttributeTargets.Class)]
 	public sealed class TreeNodeAttribute : Attribute {
-		int col_count;
-
+		bool list_only;
+		
+		[Obsolete ("This is no longer needed; it gets detected by Gtk#")]
 		public int ColumnCount {
+			get { return 0; }
+			set { }
+		}
+
+		public bool ListOnly {
 			get {
-				return col_count;
+				return list_only;
 			}
 			set {
-				col_count = value;
+				list_only = value;
 			}
 		}
 	}
